function BK = DrawFixationPt(WSS, RSS, FixProps, BK, ComSet)
%%

%% Set Defaults

DefaultColor = 255*[1 0 0 1];

%% Unpack Structs

DAQ = ComSet.DAQ;
EEG = ComSet.EEG;
NSC = ComSet.NSC;
DIO = ComSet.DIO;
Event = ComSet.Event;
code = ComSet.code;

if isempty(FixProps)
    FixColor = DefaultColor;
else
    FixColor = FixProps.FixColor;
end

%%
if nargin < 10 % Set default color to red 
    FixColor = DefaultColor;
end

switch Event
    case 1 % Block 1 event
        pin = 3;
    case 2 % Block 2 event
        pin = 5;
    case 3 % Block 3 event
        pin = 7;
    case 4 % Block 4 event
        pin = 9;
end

% Draw / Display Fixation Point              
Screen('FillRect', WSS, 128, RSS);
Screen('DrawDots', WSS, [0, 0], 10, FixColor, ...
       [RSS(3)/2 RSS(4)/2], 1);
Screen('DrawingFinished', WSS);
BK.TimeStamps(BK.j) = Screen('Flip',WSS);

if EEG == 1
    if DAQ == 1
        DIO.digitalWrite(pin,1)
        DIO.digitalWrite(pin,0)
    end
    NetStation('Event', NSC{pin-1}, BK.TimeStamps(BK.j)); % Send event to Net Station
end

disp('Fixation Point Displayed')

BK.KeyCodes(BK.j) = -1;
BK.TimeCodes(BK.j) = code;
BK.j = BK.j+1; % Advance Counter

end