function BK = DrawImage(WSS, RSS, Image, BK, ImgProps, ComSet)
% BK = DrawImage(WSS, RSS, Image, BK, ImgProps, ComSet)
%
%******************************************************************
%
% Written by Cameron Rodriguez
% Last Modified 2012/07/03
%
%******************************************************************
%
% see also: PTB4NITP, Activate_Screens, NITPParadigm, ...
%           RealTimeAnalysis_NITP, DrawText, GetKeyPress, ...
%           PlayAudio, WaitForTR
%
%******************************************************************

%% Set Defaults

DefaultCenter = [0,0];
DefaultScale = 1;

%% Unpack Structs

DAQ = ComSet.DAQ;
EEG = ComSet.EEG;
NSC = ComSet.NSC;
DIO = ComSet.DIO;
Event = ComSet.Event;
code = ComSet.code;

if isempty(ImgProps)
    ImgCenter = DefaultCenter;
    ImgScale = DefaultScale;
else
    if isfield(ImgProps, 'ImgCenter')
        ImgCenter = ImgProps.ImgCenter;
    else
        ImgCenter = DefaultCenter;
    end
    if isfield(ImgProps, 'ImgScale')
        ImgScale = ImgProps.ImgScale;
    else
        ImgScale = DefaultScale;
    end
end

%%

switch Event
    case 1 % Block 1 event
        pin = 3;
    case 2 % Block 2 event
        pin = 5;
    case 3 % Block 3 event
        pin = 7;
    case 4 % Block 4 event
        pin = 9;
end

% Draw / Display Image 
% Screen('DrawTexture', windowPointer, texturePointer ...
%        [,sourceRect] [,destinationRect] [,rotationAngle] ...
%        [, filterMode] [, globalAlpha] [, modulateColor] ...
%        [, textureShader] [, specialFlags] [, auxParameters]);

if ~isempty(ImgProps)
    scale =  1; % preallocate matrix for grating display scaling
    % turn rectangle centers to 4 corner values
    [SXC,SYC] = RectCenter(RSS);
    Xpos = SXC + ImgCenter(1); Ypos = SYC + ImgCenter(2);
    texrect = Screen('Rect', Image);
    DestRect = CenterRectOnPoint(texrect * ImgScale, SXC, SYC)';
    Screen('DrawTexture', WSS, Image, [], DestRect);
else
    Screen('DrawTexture', WSS, Image);
end
BK.TimeStamps(BK.j) = Screen('Flip',WSS);

if EEG == 1
    if DAQ == 1
        DIO.digitalWrite(pin,1)
        DIO.digitalWrite(pin,0)
    end
    NetStation('Event', NSC{pin-1}, BK.TimeStamps(BK.j)); % Send event to Net Station
end

disp('Image Displayed')

BK.TimeCodes(BK.j) = code;
BK.KeyCodes(BK.j) = -1;
BK.j = BK.j+1; % Advance Counter
end

