function TrialVariables = NITPParadigm(BlockNum, Params, ScreenHandels, MFImg, LRImg, Audio, Fsample, DIO)                                        
% TrialVariables = ...
% ExampleParadigmBlock(BlockNumber, Params, ScreenHandels, DIO)
%
%******************************************************************
%
% Written by Cameron Rodriguez
% Last Modified 2012/07/03
%
%******************************************************************
%
% see also: PTB4NITP, Activate_Screens, ...
%           RealTimeAnalysis_NITP, DrawText, GetKeyPress, ...
%           PlayAudio, DrawImage, WaitForTR
%
%******************************************************************

%% Program Settings

%% Activate Keyboard
    
KbName('UnifyKeyNames');
FlushEvents('keyDown');

space=KbName('SPACE');
esc=KbName('ESCAPE');
right=KbName('RightArrow');
left=KbName('LeftArrow');
up=KbName('UpArrow');
down=KbName('DownArrow');
shift=KbName('RightShift');

Keys.GoKey = KbName('g');
Keys.KillKey = KbName('k');
Keys.TRKey1 = KbName('t'); % TR signal key
Keys.TRKey2 = KbName('5'); % TR signal key
Keys.TRKB = KbName('5%');  % Keyboard TR

Keys.KB1 = KbName('1!');   % Keyboard 1
Keys.KB2 = KbName('2@');   % Keyboard 1
Keys.KB3 = KbName('3#');   % Keyboard 1
Keys.KB4 = KbName('4$');   % Keyboard 1

Keys.BB1 = KbName('1');    % Button Box 1
Keys.BB2 = KbName('2');    % Button Box 2
Keys.BB3 = KbName('3');    % Button Box 3
Keys.BB4 = KbName('4');    % Button Box 4

% Keep KbCheck for looking for the TR signals
% olddisabledkeys = DisableKeysForKbCheck([KbName('T'),KbName('5')]);

%% Unpack Stucts
WES = ScreenHandels.WES;   % Window Handel Experimenter
RES = ScreenHandels.RES;   % Window Rectangle Experimenter
ifiE = ScreenHandels.ifiE; % interframe interval Experimenter
WSS = ScreenHandels.WSS;   % Window Handel Subject
RSS = ScreenHandels.RSS;   % Window Rectangle Subject
ifiS = ScreenHandels.ifiS; % interframe interval Subject

EEG = Params.EEG;
DAQ = Params.DAQ;

J1Max = Params.Jitter1Max;
J1Min = Params.Jitter1Min;
ISI = Params.TrialLenght(BlockNum);
TPB = Params.TrialsPerBlock;

NSC = Params.NS_Code;

LorR = Params.LorR; 
MorF = Params.MorF;

%% Pack Structs

% Device Communication Settings
ComSet.DAQ = DAQ;
ComSet.EEG = EEG;
ComSet.NSC = NSC;
ComSet.DIO = DIO;

%% Create the ISIs
    
% Create random ISI's in the interval desired
    mu = 1;
    ISIs1 = exprnd(mu, 1, 1000 );
    ISIs1 = (ISIs1-min(ISIs1));
    ISIs1 = ISIs1/max(ISIs1);
    ISIs1 = ((J1Max-J1Min)*ISIs1 )+J1Min;
    
% round to the 1000th of a sec
    ISIs1 = (round(ISIs1*1000))/1000;

%% Preallocate Variable for Speed

% Book Keepers
BK.TimeCodes = nan(1, 1000);
BK.TimeStamps = nan(1, 1000);
BK.KeyCodes = nan(1, 1000);

WhyBlockText = cell(1, TPB(1));
MFCI = nan(1, TPB(2));
ACLD = nan(1, TPB(3));

%% Make the Textures

TMF = nan(1, numel(MFImg)); % Textures of MF images
for i = 1:size(MFImg,2) % Make the Textures of Old Images
    TMF(i)=Screen('MakeTexture', WSS, MFImg{i});
end
TLR = nan(1, numel(LRImg)); % Textures of LR images
for i = 1:numel(LRImg) % Make the Textures of Lure Images
    TLR(i)=Screen('MakeTexture', WSS, LRImg{i});
end

%% Permute the presentation

Perm1 = randperm(numel(MFImg)); % Jumble the order
Perm2 = randperm(numel(LRImg)); % Jumble the order
Perm3 = randperm(numel(Audio)); % Jumble the order

BK.j = 1;% Set inital point for counter

%% Wait for the first TR

if BlockNum == 1
    [BK, ODK] = WaitForTR(WSS, Keys, BK);
end

%% Block Starts

BK.TimeStamps(BK.j) = GetSecs;
BK.TimeCodes(BK.j) = 3; % Block Begin
BK.KeyCodes(BK.j) = -1; % No Key Press
BK.j = BK.j+1;

ComSet.Event = BlockNum; % Used to pick output for the DIO and EEG

switch Params.BlockType(BlockNum)

    case 1 % Text <<<<<<<<<<< step3 - text display

        for q=1:TPB(BlockNum)
         % Draw Text onto the screen
         
            ComSet.code = 4; % What to put in for Time Stamp code
            
            TextProps.TextColor = 255*[0 0 0]; %rgb
            %TextProps.TextColor = 255*[1 0 0]; %<<<<< step3 CHANGE COLOUR [rgb]
            %TextProps.TextColor = 255*[0 1 0]; %<<<<< step3 CHANGE COLOUR [rgb] 
            
            TextProps.TextSz = 36;              %<<<<< step3 CHANGE TEXT SIZE
            %TextProps.TextSz = 10;
            
            WhyBlockText{q} = why_CRmod;                                %<<<<< step3 edit why_CRmod.m to find out what it does
            %WhyBlockText{q} = 'This why command is very very silly.';  %<<<<< step3 - add your own text;'
            
            BK = DrawText(WSS, WhyBlockText{q}, TextProps, BK, ComSet); %<<<<< step3 edit DrawText.m to see how it works (It's cameron's file not PTB)
                
         % Wait
            %WaitSecs(3-ifiS); % time in Seconds %<<<<<< step3 ISI
            WaitSecs(1-ifiS);
            
         % Draw Fixation Point onto the screen
            ComSet.code = 13;
            %FixProps.FixColor = 255*[1 0 0 1]; % [R G B alpha] %<<<<< step3 color of fixation
            FixProps.FixColor = 255*[0 0 1 0];
            BK = DrawFixationPt(WSS, RSS, FixProps, BK, ComSet);
         % Wait
            WaitSecs(ISI-ifiS); % Wait time in Seconds
        end

    case 2 % Visual <<<<<<<<<< step4 - visual display

        for q=1:TPB(BlockNum)
         % Draw Image onto the screen
            ComSet.code = 7; % Code for TimeStamp
            % ImgProps = [];
            ImgProps.ImgCenter = [0, 0]; % [ShiftXpix, ShiftYpix]
            %ImgProps.ImgCenter = [153, 73]; <<<<<<<<<< step4
            ImgProps.ImgScale = 0.5; % Image Scale Factor
            %ImgProps.ImgScale = 0.1;        <<<<<<<<<< step4
            %ImgProps.ImgCenter = [Xcenter, Ycenter] 
            BK = DrawImage(WSS, RSS, TMF(Perm1(q)), BK, ImgProps, ComSet);  %<<<<<<<<<< step4           
         
         % Wait
            WaitSecs(ISI-ifiS); % Wait time in Seconds

         % Draw Text onto the screen                              
            ComSet.code = 5; % What to put in for Time Stamp code
            TextProps.TextColor = 255*[0 0 0];
            TextProps.TextSz = 36;
            DisplayText = 'Button 1 = Male, Button 2 = Female';
            BK = DrawText(WSS, DisplayText, TextProps, BK, ComSet); 

         % Get Response
            timeout = 3; % lenght of time to wait for response
            ComSet.code = 8; % What to put in for Time Stamp code
            [BK, TimeElapsed, Rbutton] = GetKeyPress(Keys, BK, timeout, ComSet);

            if MorF((Perm1(q))) == (Rbutton-1)
                disp('CORRECT')
                MFCI(q)= 1;
            elseif Rbutton == -1
                disp('Not Answered')
                MFCI(q)= -1;
            else
                disp('INCORRECT')
                MFCI(q)= 0;
            end

         % handle the remaining time of the responce window
            if ((timeout-TimeElapsed) > ifiS) % if there is more than a frame of time left
                ComSet.code = 14;
                FixProps.FixColor = 255*[1 0 0 1]; % [R G B alpha]
                BK = DrawFixationPt(WSS, RSS, FixProps, BK, ComSet);
                WaitSecs(timeout-TimeElapsed);
            elseif ((timeout-TimeElapsed) <= ifiS) && ((timeout-TimeElapsed) > 0)
                WaitSecs(timeout-TimeElapsed);
            end

        end

    case 3 % Audio <<<<<<<<<<< step5 - audio presentation
        for q=1:TPB(BlockNum)
         % Play some audio
            ComSet.code = 11;
            BK = PlayAudio(WSS, RSS, Audio{Perm3(q)}, Fsample(Perm3(q)), BK, ComSet);

         % Draw Text onto the screen                              
            ComSet.code = 5; % What to put in for Time Stamp code
            TextProps.TextColor = 255*[0 0 0];
            TextProps.TextSz = 36;
            DisplayText = 'Button 1 = Like, Button 2 = Dislike';
            BK = DrawText(WSS, DisplayText, TextProps, BK, ComSet); 

         % Get Response

            timeout = 3; % lenght of time to wait for response
            ComSet.code = 9; % What to put in for Time Stamp code
            [BK, TimeElapsed, Rbutton] = GetKeyPress(Keys, BK, timeout, ComSet);

            if Rbutton == 1
                ACLD(q)= 1;
                disp('LIKE')
            elseif Rbutton == 2
                ACLD(q)= 0;
                disp('DISLIKE')
            else
                ACLD(q)= -1;
                disp('No Answer')
            end

         % handle the remaining time of the responce window
            if ((timeout-TimeElapsed) > ifiS) % if there is more than a frame of time left
                ComSet.code = 15;
                FixProps.FixColor = 255*[1 0 0 1]; % [R G B alpha]
                BK = DrawFixationPt(WSS, RSS, FixProps, BK, ComSet);
                WaitSecs(timeout-TimeElapsed);
            elseif ((timeout-TimeElapsed) <= ifiS) && ((timeout-TimeElapsed) > 0)
                WaitSecs(timeout-TimeElapsed);
            end

        end           
end     
   

%% End The Block    
    
BK.TimeStamps(BK.j) = GetSecs;
BK.TimeCodes(BK.j) = -3; % Block Ends
BK.KeyCodes(BK.j) = -1;

%% Trim off the excess and pack in a stuct to pass out

BK.TimeStamps(isnan(BK.TimeStamps)) = []; 
BK.TimeCodes(isnan(BK.TimeCodes)) = [];
BK.KeyCodes(isnan(BK.KeyCodes)) = [];

if isempty(WhyBlockText{1});
    WhyBlockText = [];
end
MFCI(isnan(MFCI)) = [];
ACLD(isnan(ACLD)) = [];

% General Info
TrialVariables.TimeStamps = BK.TimeStamps;
TrialVariables.TScode = BK.TimeCodes;
TrialVariables.KeyCode = BK.KeyCodes;

% Paradigm Info
TrialVariables.WhyBlockText = WhyBlockText; % Left Right Correct Incorrect
TrialVariables.MFCI = MFCI; % Male Female Correct Incorrect
TrialVariables.ACLD = ACLD; % Audio Clip Like Dislike

%% Clean Up

ODK = DisableKeysForKbCheck([]); % Restore Key Board

end

%% Subfunctions