function PTB4NITP(TestSubject)
% PTB4NITP(TestSubject) *******************************************
%
% This program was written for a tutorial for the 2012 summer 
% sessionof the Neuro Imaging Training Program on Pyschtoolbox 3. 
% This was designed to be template from which other code can be 
% made. The PTB4NITP program acts as a skeleton from which programs
% that run the Block 
% or Jitter design are called. The function Activate_Screens is 
% called to open screen(s)in PTB. RealTimeAnalysis displays the 
% captured data as a way to do a quick and dirty quality control of
% the run.    
%
%******************************************************************
%
% Written by Cameron Rodriguez cameron.rodrigue@gmail.com
% Last Modified 2012/07/03
%
%******************************************************************
%
% see also: ExampleParadigm, Activate_Screens, ...
%           RealTimeAnalysis_NITP, Activate_Screens, ...
%           NITPParadigm, , DrawText, GetKeyPress, ...
%           PlayAudio, DrawImage, WaitForTR 
%
%******************************************************************

%% just because...

PsychJavaTrouble;


%% SET TEST MODE SPEC

TestMode = 0;
if TestMode == 1
  disp('***** Test mode enabled. No data saving. *****')
end


%% GET USER INPUTS

if nargin < 1
  TestSubject = input('Who is the Test Subject? Ex: JD ==> ', 's');
end
Params.TestSubject = TestSubject;




%% SET UP SPECS

% Description
Params.LabGroup = 'Rodriguez'; %<<<< STEP 1a <<<<<<<<<<<
Params.ExperimentName = 'PTBdemo';
Params.ExperimentPurpose = 'To Show Off the Wonders of PTB';
Params.ExperimentTask = 'Var_PTB_Fxn';

% Design
Params.Design = 'Block'; 

Params.TotalBlocks = 3; %To change total number of blocks
Params.TrialsPerBlock = [2,3,4,0];
Params.BlockType = [1 2 3];  %see NITPparadigm.m
%Params.BlockType = [2 3 1];

Params.TrialLenght = [5,5,5,0]; % Block Lenght (sec)
%Params.TrialLenght = [1,1,1,0]; % Block Length
Params.IBI = 0; % Inter Block interval in sec
Params.Jitter1Max = 3; % Max Inter-Stimulus interval (sec)
Params.Jitter1Min = 1; % Min Inter-Stimulus interval (sec)
% Params.Jitter2Max = 3; % Max Inter-Stimulus interval (sec)
% Params.Jitter2Min = 1; % Min Inter-Stimulus interval (sec)
% ect...

Params.DrawFixationPt = 1; % 1 = On, 0 = Off
Params.RTA = 1; %Real Time Analysis 1 = On, 0 = Off
Params.UseOneMonitor = 1; % Only use 1 monitor otherwise 
                          % use 2 if available and not in TestMode

% Hardware
Params.MonitorType = 'Projector'; % 'Goggles';
Params.EEG = 0; % using the EGI EEG system Simultainiouly? 
                % No = 0 or Yes = 1
Params.DAQ = 0; % using the Arduino DAQ? No = 0 or Yes = 1
Params.NetStationPort = 55513; % Net Station Port ID
Params.NetStationIP = '169.254.223.206'; % Net Station IP

% DO YOU HAVE MORE SPECS TO ADD - JUST FOLLOW THE ABOVE TEMPLATE
% Params.YourParameterHere = [];



%% Unpack The Param Struct to save on typing

DrawFixationPt = Params.DrawFixationPt; 
RTA = Params.RTA; 
UseOneMonitor = Params.UseOneMonitor; 
TrialLenght = Params.TrialLenght;
IBI = Params.IBI;
DAQ = Params.DAQ;
EEG = Params.EEG;



%% Constants

Constants.SSE = 0; % Location in deg of left screen center wrt the
                   % test subject's fixation
Constants.ESE = 0; % Location in deg of Right screen center wrt the
                   % Experimenter's fixation
Constants.XOffset = 0; % horizontal stimulus position 
                       % (-)left, (+)right
Constants.YOffset = 0; % vertical stimulus position
                       % (-)up, (+)down
Constants.Computer = Screen('Computer'); % Get The Computer ID
Constants.PTBVersion = Screen('Version'); % Get The PTB Version

Params.Constants = Constants;

%% Initialize SaveFile Variables

% Store the Variables for all trials (VAT) 
VAT.TimeStamps = []; % Program Timing
VAT.TScode  = []; % Program Timing Codes
VAT.KeyCode  = [];  % Program Key Codes

% Variable(s) Specific to your paradigm
VAT.WhyBlockText = []; % Text Displayed from Why Fxn
VAT.MFCI = []; % Male Female Correct Incorrect
VAT.ACLD = []; % Audio Clip Like Dislike

% etc...

% TScode | Event
Params.TScodeIDTable = {...
  ['   2 | EEG Recording Start'];...
  ['  -2 | EEG Recording End'];...
  ['   1 | Scan Begins (First TR recorded)'];...
  ['  -1 | Matlab Stops Recording Events'];...
  ['   3 | Block Starts'];...
  ['  -3 | Block Ends'];...
  ['   4 | Why Fxn Text Displayed'];...
  ['   5 | Responce Text Displayed'];...
  ['   6 | LR Image Displayed'];...
  ['   7 | MF Image Displayed'];...
  ['   8 | Responce Button Pressed MF'];...
  ['   9 | Responce Button Pressed Audio'];...
  ['  10 | Not Used'];...
  ['  11 | Audio File Begins Playing'];...
  ['  12 | Gray Screen Displayed']
  ['  13 | Text Block Fixation Point Displayed'];...
  ['  14 | Early Responce MF Image Block Fixation Point Displayed'];...
  ['  15 | Early Responce Audio Fixation Point Displayed']};


Params.NS_Code = {['EVT1'];... % Event 1
                  ['EVT2'];... % Event 2
                  ['EVT3'];... % Event 3
                  ['EVT4'];... % Event 4
                  ['EVT5'];... % Event 5
                  ['EVT6'];... % Event 6
                  ['EVT7'];... % Event 7
                  ['EVTn']};   % Event ...
Params.Duration = [0; 0; 0; 0; 0; 0; 0; 0]; % Event durations in sec, matched to NS_Code 

%% Establish Output Files

switch Params.LabGroup
    case 'Rodriguez'
        filename  = [Params.LabGroup, '_', ...
             Params.ExperimentName, '_', ...
             datestr(now,'yyyymmdd_HHMMSS'), ...
             '_SUBJ_', TestSubject];
        if IsWin
            data_folder = ' ';
            backup_data_folder = ' ';
            image_folder = ' ';
        else
            data_folder = '/Users/Cameron/Documents/MATLAB/Data/';
            backup_data_folder = ...
                '/Users/Cameron/Documents/MATLAB/BackupData/'; ...
            image_folder = '/Users/Cameron/Documents/MATLAB/NITP PTB/';

        end
    case 'YOURGROUPNAME' %<<<<<< STEP 1b <<<<<<<<<<<
        filename  = TestSubject;
        if IsWin
            data_folder = ' ';
            backup_data_folder = ' ';
            image_folder = ' ';
        else
%             data_folder = '/Users/al/Work/Tools/PsychtoolboxTutorial/Step3/Data/';
%             backup_data_folder = ...
%                 '/Users/al/Work/Tools/PsychtoolboxTutorial/Step3/Data/'; ...
%             image_folder = '/Users/al/Work/Tools/PsychtoolboxTutorial/Step3/';
        end  
end         
SaveFolder.data_folder = data_folder;
SaveFolder.backup_data_folder = backup_data_folder; 
SaveFolder.filename = filename;

%% Activate Keyboard
    
KbName('UnifyKeyNames');
FlushEvents('keyDown');

Keys.GoKey = KbName('g');
Keys.KillKey = KbName('k');
Keys.TRKey1 = KbName('t'); % TR signal key
Keys.TRKey2 = KbName('5'); % TR signal key
Keys.TRKB = KbName('5%');  % Keyboard TR

Keys.KB1 = KbName('1!');   % Keyboard 1
Keys.KB2 = KbName('2@');   % Keyboard 1
Keys.KB3 = KbName('3#');   % Keyboard 1
Keys.KB4 = KbName('4$');   % Keyboard 1

%%%%%%%%%%%%%%%%%%%%%%%%%
%**  Button Box Mode  **%
%%%%%%%%%%%%%%%%%%%%%%%%%
%                       %
% USB                   %
% HHSC-1x5-D            %
% HID Key NAR 12345 003 %
%                       %
%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%

Keys.BB1 = KbName('1');    % Button Box 1
Keys.BB2 = KbName('2');    % Button Box 2
Keys.BB3 = KbName('3');    % Button Box 3
Keys.BB4 = KbName('4');    % Button Box 4

% Code to be used to later on diable the MRI TR trigger Keys
% olddisabledkeys=DisableKeysForKbCheck([KbName('T'), KbName('5')])

% Code to be used to later on restore the MRI TR trigger Keys
% olddisabledkeys=DisableKeysForKbCheck([])

%% Initialize the random stream

RandStream.setDefaultStream(...
                   RandStream('mt19937ar','seed',sum(100*clock)));


%% Load the images to display

LR_ImgFile = [image_folder,'MooneyImgLR.mat'];
MF_ImgFile = [image_folder,'MooneyImgMF.mat'];

Params.LR_ImgFile = LR_ImgFile;
Params.MF_ImgFile = MF_ImgFile;

load(LR_ImgFile);
% File Contents: 
% ImageNames = Left-Right File Names
% Images = Left-Right Images
% LRbinary,  0 == Left Facing, 1 == Right Facing
LRImg = Images;
Params.LRImgNames  = ImageNames;
Params.LorR = LRbinary;
clear Images; clear ImageNames; clear LRbinary;

load(MF_ImgFile);
% File Contents: 
% ImageNames = Male-Female File Names
% Images =  Male-Female Images
% MFbinary,  0 == Male, 1 == Female
MFImg = Images;
Params.MFImgNames  = ImageNames;
Params.MorF = MFbinary;
clear Images; clear ImageNames;

%% Load the Audio

audioFiles = cell(1,5);
audioFiles{1} = 'splat';
audioFiles{2} = 'handel';
audioFiles{3} = 'laughter';
audioFiles{4} = 'train';
audioFiles{5} = 'gong';
Params.AudioFiles = audioFiles;

Audio = cell(1,numel(audioFiles));
Fsample = nan([1,numel(audioFiles)]);

for k=1:numel(audioFiles)
    load(audioFiles{k});
    Audio{k} = y; Fsample(k) = Fs;
    clear y; clear Fs;
    % sound(Audio{k}, Fsample(k)) 
end    

%% Initialize the DAQ
    
if DAQ == 1
    DIO = arduino('/dev/tty.usbmodem621');
    for i = 2:19
        DIO.pinMode(i,'output'); DIO.digitalWrite(i,0)
    end
else
    DIO = [];
end
    
%% Activate/Check the EEG capture

% doc NetStation  
if EEG == 1
    
    Params.NScode = NS_Code;
    Params.NSduration = NS_Duration;

    NSPT = Params.NetStationPort; % Done for formatting
    NSIP = Params.NetStationIP; % Done for formatting
    [status, error] = NetStation('Connect', NSIP, NSPT);
    
    if status ~= 0 % Kill Program
        disp(error); return;
    end

    NetStation('Synchronize'); NetStation('StartRecording');
    VAT.TimeStamps = [VAT.TimeStamps, GetSecs];
    VAT.TimeCode  = [VAT.TScode, 2];
    VAT.KeyCode    = [VAT.KeyCode, -1];
    
end

%% OPEN A SCREEN AND SAVE PARAMETERS FOR LATER USE

    [ScreenHandels, Screen_Parameters, PPD_DPP] = ... 
        Activate_Screens(Constants, Params); %<<<<<<<<<<<<< Step 2 <<<<<<<
    WES = ScreenHandels.WES; % Window Handel Right Screen
    RES = ScreenHandels.RES; % Window Rectangle Right Screen
    WSS = ScreenHandels.WSS; % Window Handel Left Screen
    RSS = ScreenHandels.RSS; % Window Rectangle Left Screen
    
    Params.Screen_Parameters = Screen_Parameters;
    Params.PPD_DPP = PPD_DPP;

%% Run Experiment

try % Start Try - Catch
      
    %% Run The Display loop
    
    for i = 1:Params.TotalBlocks

        %% Run The Paradigm

        TrialVariables = NITPParadigm(i, Params, ScreenHandels, ...
                         MFImg, LRImg, Audio, Fsample, DIO);

        %% Concatinate the current trial data with previous data

        % General Info
        VAT.TimeStamps = [VAT.TimeStamps, TrialVariables.TimeStamps];
        VAT.TScode = [VAT.TScode, TrialVariables.TScode];
        VAT.KeyCode = [VAT.KeyCode, TrialVariables.KeyCode];

        % Paradigm Info
        VAT.WhyBlockText = TrialVariables.WhyBlockText; % Why Block Text
        VAT.MFCI = [VAT.MFCI, TrialVariables.MFCI]; % Male Female Correct Incorrect
        VAT.ACLD = [VAT.ACLD, TrialVariables.ACLD]; % Audio Clip Like Dislike
        %% Save Data after each set

        if TestMode == 0
            save([data_folder,filename,'_set',num2str(i),'.mat'], ...
                'Params', 'TrialVariables');
            save([backup_data_folder,filename,'_set',num2str(i),'.mat'], ...
                'Params', 'TrialVariables');
        end

        %% Peek at the Data after each set

        if RTA == 1        
            RealTimeAnalysis_NITP(VAT, Params, MFImg, LRImg, Audio, Fsample);
        end
                
    end
        

    %% Stop Recording & Disconnnect the Netstation
    
    if EEG == 1
        WaitSecs(2); % for some padding of the EEG data
        NetStation('StopRecording');
        NetStation('Disconnect');
        VAT.TimeStamps = [VAT.TimeStamps, GetSecs];
        VAT.TScode     = [VAT.TScode, -2];
        VAT.KeyCode    = [VAT.KeyCode,-1];
    end
   
%% Save The Raw Data   

   % End Event Capture
    VAT.TimeStamps = [VAT.TimeStamps, GetSecs];
    VAT.TScode     = [VAT.TScode, -1];
    VAT.KeyCode    = [VAT.KeyCode,-1];
    if TestMode == 0
        save([data_folder,filename,'.mat'], ...
            'Params', 'VAT');
        save([backup_data_folder,filename,'.mat'], ...
            'Params', 'VAT');
    end

%% All Done Time For Fun

    DrawFormattedText(WSS, 'ALL DONE! THANK YOU!', ...
                      'center', 'center', 0 , 45);
    Screen('Flip',WSS);
    WaitSecs(2);

%% Do some clean up
    
    Screen('CloseAll'); % Close PTB windows
    ShowCursor;   
    if DAQ == 1 % Close the DAQ if open
        delete(DIO);
    end
        
%% Run Some Quick Stats % Analysis
    
    if RTA == 1        
       RealTimeAnalysis_NITP(VAT, Params, MFImg, LRImg, Audio, Fsample);
    end
    
catch lasterr
    % this "catch" section executes in case of an error in the
    % "try" section above.  The error is captured in "lasterr". 
    % Importantly, it closes the onscreen window(s).
    
    Screen('CloseAll');
    
    % Save what you can
    if TestMode == 0    
       save([data_folder,'CrashSave_',filename,'.mat'],...
            'Params', 'VAT', 'lasterr');
       save([backup_data_folder,'CrashSave_',filename,'.mat'], ...
            'Params', 'VAT', 'lasterr');      
    end
    
    % Stop Recording & Close the EEG 
    if EEG == 1
        WaitSecs(2); % for some padding of the EEG data
        NetStation('StopRecording');
        NetStation('Disconnect');
        VAT.TimeStamps = [VAT.TimeStamps, GetSecs];
        VAT.TScode     = [VAT.TScode, -2];
        VAT.KeyCode    = [VAT.KeyCode,-1];
    end
    
    % Restore the Keyboard
    DisableKeysForKbCheck([]); 
    
    % Show the Cursor
    ShowCursor;
    
    rethrow(lasterr)
    
    % Close the DAQ if open
    if DAQ == 1 
        delete(DIO);
    end
    
    
end % End Try - Catch

    % Restore the Keyboard
    DisableKeysForKbCheck([]); 
    ShowCursor;
    
    % Clear the Buffers
    if DAQ == 1
        delete(DIO);
    end
    clear all;
end % All Done, That's All He or She Wrote

%% Subfunctions