function RealTimeAnalysis_NITP(VAT, Params, MFImg, LRImg, Audio, Fsample)
% RealTimeAnalysis(VAT, Params)
%
% This program was written to give a quick peek at the data 
% recorded from the display computer.  This can be used to make
% sure everything is recording properly
%
%******************************************************************
%
% Written by Cameron Rodriguez
% Last Modified 2012/07/03
%
%******************************************************************
%
% see also: PTB4NITP, ExampleParadigm, Activate_Screens, ...
%           Activate_Screens, NITPParadigm, , DrawText, ...
%           GetKeyPress, PlayAudio, DrawImage, WaitForTR 
%
%*****************************************************************

%% Unpack the Structures

TimeStamps = VAT.TimeStamps;
TScode = VAT.TScode;
KeyCode = VAT.KeyCode;

EEG = Params.EEG;

MRStInds = find(TScode == 1, 1);
MRStInd = min(MRStInds);
MREndInds = numel(TScode);
MREndInd = max(MREndInds);

if EEG == 1
    EEGStInds = find(TScode==2);
    EEGStInd = min(EEGStInds);
    EEGEndInds = find(TScode==-2,1);
    EEGEndInd = max(EEGEndInds);
    if (isempty(EEGStInds) | isempty(EEGEndInds))
        EEG = 0; % Missing an EEG timepoint (ie scan not finished)
    end
end
if EEG~=1
    EEGStInds = []; EEGStInd = [];
    EEGEndInds = []; EEGEndInd = [];
end

%% Display the Time Stamp Code lookup table

disp(Params.TScodeIDTable)

%% Look at the Time Stamps and Codes recorded

% Time from the Start on the MRI clock
TimeFromStartMRI = TimeStamps-TimeStamps(MRStInd); 
if EEG == 1 % Time from the Start on the EEG clock
    TimeFromStartEEG = TimeStamps-TimeStamps(EEGStInd); 
    disp('Assuming a sampling rate for the EEG of 250Hz');
    SamplesFromStart = 250*TimeFromStartEEG;
    EEGTimeLength = TimeStamps(EEGEndInd)-TimeStamps(EEGStInd);
end

BlockStInds =  find(TScode ==  3);
BlockEndInds = find(TScode == -3);

MRTimeLength = TimeStamps(MREndInd)-TimeStamps(MRStInd);

disp('***********************************************************')
disp('***********************************************************')

disp('The Display Computers Data File Contains:')
disp(' ')
disp(['   ', num2str(numel(EEGStInds)),...
      ' Netstation Record Start Point(s)'])
disp(['   ', num2str(numel(EEGEndInds)),...
      ' Netstation Record End Point(s)'])

disp(['   ', num2str(numel(BlockStInds)),' Block Start Point(s)'])
disp(['   ', num2str(numel(BlockEndInds)),' Block End Point(s)'])
disp(' ')
disp('***********************************************************')
disp(' ')
disp(['MRI Scan Time Length: ', ...
      num2str(MRTimeLength), ' sec'] )
if EEG == 1
disp(['EEG Time Record Time Length: ', ...
      num2str(EEGTimeLength),' sec'])
end
disp(' ')
disp('***********************************************************')
disp('***********************************************************')
    
figure;
% Plot Time From MRI Start Versus the Time Stamp Index
subplot(2,1,1)
    plot(TimeFromStartMRI); 
    xlabel('Time Stamp #');
    ylabel('Time From First Recieved TR Pulse (Sec)');
    axis 'tight'

% Plot Event Codes Versus the Time Stamp Index
subplot(2,1,2)
    plot(TScode);
    ylabel('Event Code');
    xlabel('Time Stamp Index');
    axis 'tight'
         
 